""" Function for generating list of seeds for parallel COMPAS runs.
George Howitt 05-06-2017"""

def seedgen(initial_seed,N_procs,N_binaries):
    #  initial seed is the randomSeed of the first run
    #  N_procs is the number of batches
    #  N binaries is the total nr of binaries over combined batches

        assert float(N_binaries/N_procs)%1 == 0

        steps_per_proc = N_binaries/N_procs
        seed = initial_seed

        output_file = open("RandomSeed.txt","w")
        output_file.write("%d \n" % seed )
        for i in range(N_procs-1):
                seed += steps_per_proc
                output_file.write("%d \n" % seed)

        output_file.close()

#  adjust the function inputs in the line below
seedgen(0,100, 1e9)

#  when using Adaptive Importance Sampling / STROOPWAFEL algorithm: use N_procs = 2 * n_batches and N_binaries *2 (use twice the nr of total Binaries sampled for N_binaries to reproduce seeds)



import numpy as np
import os
import subprocess
import csv
from .classes import Location
from .constants import *
import math

def generate_grid(locations, filename):
    """
    Function which generated a txt file with the locations specified
    IN:
        locations (list[Location]) : list of locations to be printed in the file
        filename (string) : filename to save
    OUT:
        generates file with name filename with the given locations and saves to the disk
    """
    header = []
    grid = []
    for location in locations:
        current_location = []
        for key, value in location.dimensions.items():
            if key.should_print:
                if len(grid) == 0:
                    header.append(key.name)
                current_location.append(value)
        for key, value in location.properties.items():
            if key in ['generation', 'gaussian']:
                continue
            if len(grid) == 0:
                header.append(key)
            current_location.append(value)
        grid.append(current_location)
    DELIMITER = ', '
    np.savetxt(filename, grid, fmt = "%s", delimiter = DELIMITER, header = DELIMITER.join(header), comments = '')

#copied from stack overflow
def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '|', autosize = False):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        autosize    - Optional  : automatically resize the length of the progress bar to the terminal window (Bool)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    styling = '%s |%s| %s%% %s' % (prefix, fill, percent, suffix)
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print('\r%s' % styling.replace(fill, bar), end = '\r')
    # Print New Line on Complete
    if iteration >= total:
        print()

def print_samples(samples, filename, mode):
    """
    Function that prints all the hits to a file
    IN:
        samples(list(Location)): All the samples that need to be printed
        filename (String) : The filename that will be saved
        mode (Character) : Append mode / write mode / read mode, which mode to open the file with
    """
    with open(filename, mode) as file:
        for sample in samples:
            current_dict = {}
            for dimension in sorted(sample.dimensions.keys(), key = lambda d: d.name):
                current_dict[dimension.name] = sample.dimensions[dimension]
            for prop in sorted(sample.properties.keys()):
                current_dict[prop] = sample.properties[prop]
            writer = csv.DictWriter(file, current_dict.keys())
            if file.tell() == 0:
                writer.writeheader()
            writer.writerow(current_dict)

def read_samples(filename, dimensions, only_hits = False):
    """
    Function that reads samples from a given file and convert it to Location objects
    IN:
        filename (String) : The filename that will be read
        dimensions (List(Dimension)) : List of dimensions to make variables
        only_hits (Boolean) : If true it will only return the Locations which are target hits
    OUT:
        locations (List(Location)) : List of locations generated
    """
    with open(filename, newline = '') as file:
        samples = csv.DictReader(file)
        dimensions_hash = dict()
        for dimension in dimensions:
            dimensions_hash[dimension.name] = dimension
        locations = []
        for sample in samples:
            if only_hits and int(sample['is_hit']) == 0:
                continue
            sample.update((k, float(v)) for k, v in sample.items())
            locations.append(Location.create_location(dimensions_hash, sample))
        return locations

def generate_slurm_file(command, batch_num, output_folder):
    """
    Function that generates a slurm file to run the given command on helios batch
    IN:
        command (String) : The command to be run
        batch_bum (int) : Batch number, would be used to generate file name
        output_folder (Path) : Where to generate the output
    OUT:
        slurm_file (String) : Path of the file generated
    """
    slurm_folder = get_or_create_folder(output_folder, 'slurms')
    log_folder = get_or_create_folder(output_folder, 'logs')
    slurm_file = slurm_folder + "/slurm_" + str(batch_num) + ".sh"
    log_file = log_folder + "/log_" + str(batch_num) + ".txt"
    writer = open(slurm_file, 'w')
    writer.write("#!/bin/bash\n")
    writer.write("#SBATCH --mem-per-cpu=1024\n")
    writer.write("#SBATCH --output=output.out\n")
    writer.write(command + " > " + log_file + " \n")
    writer.close()
    return slurm_file

def run_code(command, batch_num, output_folder, debug = False, run_on_helios = True):
    """
    Function that runs the command specified on the command shell.
    IN:
        command list(String): A list of commands to be triggered along with the options
        batch_num (int) : The current batch number to generate the filename
        output_folder (Path) : Where to generate the output
        debug (Boolean) : If true will print stuff to console
        run_on_helios (Boolean) : If true will generate slurm file to run on helios
    OUT:
        subprocess : An instance of subprocess created after running the command
    """
    if command != None:
        if not debug:
            stdout = subprocess.PIPE
            stderr = subprocess.PIPE
        else:
            stdout = stderr = None
        command_to_run = " ".join(str(v) for v in command)
        if run_on_helios:
            slurm_file = generate_slurm_file(" ".join(str(v) for v in command), batch_num, output_folder)
            command_to_run = "sbatch -W -Q " + slurm_file
        else:
            log_folder = get_or_create_folder(output_folder, 'logs')
            log_file = log_folder + "/log_" + str(batch_num) + ".txt"
            command_to_run = command_to_run + " > " + log_file
        process = subprocess.Popen(command_to_run, shell = True, stdout = stdout, stderr = stderr)
        return process

def get_slurm_output(output_folder, batch_num):
    """
    Reads the output generated by the slurm run
    IN:
        output_folder (Path) : Where the output resides
        batch_num (int) : Will be used to generate file name
    OUT:
        (String) : The output returned by slurm run
    """
    try:
        log_folder = os.path.join(output_folder, 'logs')
        log_file = log_folder + "/log_" + str(batch_num) + ".txt"
        with open(log_file) as f:
            return_list = [float(line.rstrip()) for line in f]
            return return_list
    except:
        pass

def get_or_create_folder(path, name):
    """
    Utility function to create a folder at a given path and name or return it if it already exists
    path (String) : Path where it is created
    name (String) : Name of the folder
    """
    folder = os.path.join(path, name)
    if not os.path.exists(folder):
        os.makedirs(folder)
    return folder

def print_distributions(filename, distributions):
    with open(filename, 'w') as file:
        for distribution in distributions:
            current_dict = dict()
            for dimension, value in distribution.mean.dimensions.items():
                current_dict[dimension.name + '_mean'] = value
            for dimension, value in distribution.sigma.dimensions.items():
                current_dict[dimension.name + '_sigma'] = value
            writer = csv.DictWriter(file, current_dict.keys())
            if file.tell() == 0:
                writer.writeheader()
            writer.writerow(current_dict)

def print_logs(output_folder, log_string, log_value):
    with open(output_folder + '/log.txt', 'a') as file:
        file.write("%s = %f\n"%(log_string, log_value))

def get_zams_radius(mass, metallicity):
    metallicity_xi = math.log10(metallicity / ZSOL)
    radius_coefficients = []
    for coeff in R_COEFF:
        value = 1
        total = 0
        for series in coeff:
            total += series * value
            value *= metallicity_xi
        radius_coefficients.append(total)
    top = radius_coefficients[0] * pow(mass, 2.5) + radius_coefficients[1] * pow(mass, 6.5) \
        + radius_coefficients[2] * pow(mass, 11) + radius_coefficients[3] * pow(mass, 19) \
        + radius_coefficients[4] * pow(mass, 19.5)
    bottom = radius_coefficients[5] + radius_coefficients[6] * pow(mass, 2) \
        + radius_coefficients[7] * pow(mass, 8.5) + pow(mass, 18.5) \
        + radius_coefficients[8] * pow(mass, 19.5)
    radius = top / bottom
    return radius * R_SOL_TO_AU

def calculate_roche_lobe_radius(mass1, mass2):
    q = mass1 / mass2
    return 0.49 / (0.6 + pow(q, -2.0 / 3.0) * math.log(1.0 + pow(q, 1.0 / 3.0)))

def inverse_back(dimension, inverse):
    norm_factor = (ALPHA_IMF + 1) / (pow(dimension.max_value, ALPHA_IMF + 1) - pow(dimension.min_value, ALPHA_IMF + 1))
    return norm_factor / pow(inverse * (pow(norm_factor / dimension.max_value, 1 / -ALPHA_IMF) \
        - pow(norm_factor / dimension.min_value, 1 / -ALPHA_IMF)) \
        + pow(norm_factor / dimension.min_value, 1 / -ALPHA_IMF), -ALPHA_IMF)



from abc import ABC, abstractmethod
from scipy.stats import multivariate_normal
from .classes import Location
import numpy as np
from .utils import *
import json
from .constants import *
from . import sampler as sp

class NDimensionalDistribution:
    """
    This is a parent class for any NDimensional distribution. Any subclass of this class, must implement all the methods
    defined in this class, for example the run_sampler method.
    Examples of NDimensionalDistribution are Gaussian class and InitialDistribution class. (See below)
    """
    def __init__(self, dimensions):
        self.dimensions = dimensions

    @abstractmethod
    def run_sampler(self, num_samples):
        #This method must be implemented by all the sub classes
        pass

    @abstractmethod
    def calculate_probability_of_locations_from_distribution(self, locations):
        #This method must be implemented by all the sub classes
        pass

class InitialDistribution(NDimensionalDistribution):
    """
    This class inherits from NDimensionalDistribution. It will be used during the exploration phase to draw from Sampler class of Dimensions.
    """
    def __init__(self, dimensions):
        self.dimensions = dimensions

    def run_sampler(self, num_samples):
        """
        This function tells the class how to draw the samples.
        IN:
            num_samples (int) : How many samples to draw
        OUT:
            (list(Location), list(bool)) : A pair of lists (both of size num_samples), the first describing the samples drawn, and the second telling which of the samples are within the bounds of the respective variables
        """
        mask = np.ones(num_samples, dtype = bool)
        headers = sorted(self.dimensions, key = lambda d: d.name)
        samples = []
        for dimension in headers:
            current_samples = dimension.run_sampler(num_samples)
            mask &= dimension.is_sample_within_bounds(current_samples)
            samples.append(current_samples)
        samples = list(map(list, zip(*samples)))
        locations = [Location(dict(zip(headers, row)), {}) for row in samples]
        return (locations, mask)

    def calculate_probability_of_locations_from_distribution(self, locations):
        pass

    def calculate_rejection_rate(self, update_properties, rejected_systems, dimensions):
        num_samples = int(TOTAL_REJECTION_SAMPLES)
        (locations, mask) = self.run_sampler(num_samples)
        rejected = num_samples - np.sum(mask)
        locations = np.asarray(locations)[mask]
        [location.revert_variables_to_original_scales() for location in locations]
        update_properties(locations, dimensions)
        rejected += rejected_systems(locations, dimensions)
        return rejected / num_samples

class Gaussian(NDimensionalDistribution):
    """
    This class inherits from NDimensionalDistribution. It will be used during the refinement phase to draw adapted distributions.
    mean (Location) : The mean location of the gaussian. It is of Location type and therefore stores all the dimension
    sigma (Location) : The sigma of the gaussian. Also of Location class type
    kappa (float) : An independent scaling factor describing how wide the gaussian can be.
    """
    def __init__(self, mean, sigma = None, rejection_rate = 0, alpha = 1, kappa = 1, cov = None):
        self.mean = mean
        self.sigma = sigma
        self.kappa = kappa
        if cov == None:
            if sigma != None:
                self.cov = np.diagflat(np.power(np.asarray(self.sigma.to_array()), 2))
            else:
                raise Exception("Either sigma or cov value is needed")
        else:
            self.cov = cov
        self.rejection_rate = rejection_rate
        self.biased_weight = 1
        self.alpha = alpha
    """
        This function tells the class how to draw the samples for this class
        IN:
            num_samples (int) : How many samples to draw
            consider_rejection_rate (bool) : If we should consider rejection rate while drawing samples. If True, it will draw more samples (based on rejection_rate), such that the num_samples are within the bounds
        OUT:
            (list(Location), list(bool)) : A pair of lists (both of size num_samples), the first describing the samples drawn, and the second telling which of the samples are within the bounds of the respective variables
        """
    def run_sampler(self, num_samples, dimensions, consider_rejection_rate = False):
        if consider_rejection_rate == True:
            if self.rejection_rate == 1:
                return ([], [])
            num_samples = int(2 * np.ceil(num_samples / (1 - self.rejection_rate)))
        num_samples = int(num_samples * self.biased_weight)
        mask = np.ones(num_samples, dtype = bool)
        current_samples = multivariate_normal.rvs(mean = self.mean.to_array(), cov = self.cov, size = num_samples)
        headers = sorted(dimensions, key = lambda d: d.name)
        for index, dimension in enumerate(headers):
            mask &= dimension.is_sample_within_bounds(current_samples[:, index])
        locations = [Location(dict(zip(headers, row)), {}) for row in current_samples]
        return (locations, mask)

    @classmethod
    def draw_distributions(self, hit_locations, average_density_one_dim, kappa = 1):
        """
        Function that draws gaussians at the hits
        IN:
            hit_locations (list(Location)) : the locations which have hits to draw the gaussians
            average_density_one_dim (float) : The average density of hits to be expected per dimension
        OUT:
            gaussians (list(Gaussian)) : list of gaussians drawn
        """
        gaussians = []
        for hit in hit_locations:
            sigma = dict()
            for variable, val in hit.dimensions.items():
                sigma[variable] = self.__calculate_sigma(average_density_one_dim, variable, val)
            gaussians.append(Gaussian(hit, sigma = Location(sigma, {}), kappa = kappa))
        return gaussians

    @classmethod
    def __calculate_sigma(self, average_density_one_dim, dimension, value):
        if dimension.sampler.__name__ == sp.kroupa.__name__:
            norm_factor = (ALPHA_IMF + 1) / (pow(dimension.max_value, ALPHA_IMF + 1) - pow(dimension.min_value, ALPHA_IMF + 1))
            inverse_value = (pow(norm_factor / value, 1 / -ALPHA_IMF) - pow(norm_factor / dimension.min_value, 1 / -ALPHA_IMF)) / \
            (pow(norm_factor / dimension.max_value, 1 / -ALPHA_IMF) - pow(norm_factor / dimension.min_value, 1 / -ALPHA_IMF))
            right_distance = abs(inverse_back(dimension, inverse_value + average_density_one_dim) - value)
            left_distance = abs(inverse_back(dimension, inverse_value - average_density_one_dim) - value)
            return max(right_distance, left_distance)
        else:
            return average_density_one_dim / dimension.prior(dimension, value)

    """
    Calculates the rejection rate of each of the gaussians in batches
    IN:
        gaussians (List(Gaussian)) : The gaussians to calculate the rejection rate
    """
    @classmethod
    def calculate_rejection_rate(self, gaussians, update_properties, rejected_systems, dimensions):
        rejected = 0
        num_samples = int(REJECTION_SAMPLES_PER_BATCH)
        for index, gaussian in enumerate(gaussians):
            (locations, mask) = gaussian.run_sampler(num_samples, dimensions)
            rejected += num_samples - np.sum(mask)
            locations = np.asarray(locations)[mask]
            [location.revert_variables_to_original_scales() for location in locations]
            update_properties(locations, dimensions)
            rejected += rejected_systems(locations, dimensions)
        return rejected / (num_samples * len(gaussians))

    """
    Given a list of locations, calculates the probability of drawing each location from the given gaussian
    IN:
        locations (list(Location)) : list of locations
    """
    @classmethod
    def calculate_probability_of_locations_from_distribution(self, locations, gaussians):
        samples = []
        for location in locations:
            samples.append(location.to_array())
        pdfs = np.zeros(len(locations))
        for index, gaussian in enumerate(gaussians):
            pdf = multivariate_normal.pdf(samples, gaussian.mean.to_array(), gaussian.cov, allow_singular = True)
            pdf = pdf * gaussian.biased_weight
            pdfs += pdf
        for index, location in enumerate(locations):
            location.properties['q_pdf'] = pdfs[index]



from . import sampler as sp
import numpy as np

class Dimension:
    """
    A class that represents one of the dimensions that we will stroopwafelize
    name (string): The name of this dimension. This will be used to print in the file
    min_value (float): The minimum value this dimension can take
    max_value (float): The maximum value this dimension can take
    sampler (Sampler): type of sampling for this dimension during the exploration phase. It should be from the class Sampler
    prior (Prior): A function that will be used to calculate the prior for this dimension
    should_print (bool): If this dimension should be printed to the file
    """
    def __init__(self, name, min_value = 0, max_value = 0, sampler = None, prior = None, should_print = True):
        self.name = name
        self.min_value = min_value
        self.max_value = max_value
        self.sampler = sampler
        self.prior = prior
        self.should_print = should_print
        if self.sampler.__name__ == sp.flat_in_log.__name__:
            self.min_value = np.log10(min_value)
            self.max_value = np.log10(max_value)
        elif self.sampler.__name__ == sp.uniform_in_sine.__name__:
            self.min_value = -1
            self.max_value = 1
        elif self.sampler.__name__ == sp.uniform_in_cosine.__name__:
            self.min_value = -1
            self.max_value = 1
    """
    Function that samples the variable based on the given sampler class
    IN:
        num_samples (int) : number of samples required
    OUT:
        list(float) : samples
    """
    def run_sampler(self, num_samples):
        return self.sampler(num_samples, x = self.min_value, y = self.max_value)

    """
    Function that returns which samples are within the bounds of this variables
    IN:
        sample (list(float)) : a list of points which we need to check if in bounds
    OUT:
        list(bool) : which of the given samples are within the bounds
    """
    def is_sample_within_bounds(self, samples):
        return (samples >= self.min_value) & (samples <= self.max_value)

    def __str__(self):
        return self.name

class Location:
    """
    Describe a point in N-Dimensional space
    dimensions (dict(Dimension : float)) : dictionary of mapping between class Dimension and its float value
    properties (dict) : A location could have more properties which we dont want to stroopwafelize but still store. For example, metallicity_2 (same as metallicity_1) or ID from Compas
    """
    def __init__(self, dimensions, properties):
        self.dimensions = dimensions
        self.properties = properties

    """
    This method creates a location instance when supplied with a dimension hash and the row of samples. 
    Each key of the row which corresponds to a dimension goes to the dimensions property and the rest goes to the properties property
    IN:
        dimension_hash(dict(string, Dimension)) : Dictionary having key as a name of Dimension and mapping it to the Dimension object
        row_hash(dict(string, float)) : Row we want to hash to
    """
    @classmethod
    def create_location(cls, dimensions_hash, row_hash):
        dimensions = dict()
        properties = dict()
        for key, value in row_hash.items():
            if key in dimensions_hash:
                dimensions[dimensions_hash[key]] = value
            else:
                properties[key] = value
        return Location(dimensions, properties)

    """
    Converts the current object of Location class to an array sorted by the key name
    """
    def to_array(self):
        array = []
        for dimension in sorted(self.dimensions.keys(), key = lambda d: d.name):
            array.append(self.dimensions[dimension])
        return array

    def __str__(self):
        string = ''
        for dimension in self.dimensions.keys():
            string += dimension.name + " : " + str(self.dimensions[dimension]) + ","
        return string

    """
    Converts back each value of the location to the original scale defined in the interface
    """
    def revert_variables_to_original_scales(self):
        for dimension, value in self.dimensions.items():
            if dimension.sampler.__name__ == sp.flat_in_log.__name__:
                self.dimensions[dimension] = np.power(10, value)
            elif dimension.sampler.__name__ == sp.uniform_in_sine.__name__:
                self.dimensions[dimension] = np.arcsin(value)
            elif dimension.sampler.__name__ == sp.uniform_in_cosine.__name__:
                self.dimensions[dimension] = np.arccos(value) - np.pi / 2

    """
    Converts each value of the location to the new transformed scale
    """
    def transform_variables_to_new_scales(self):
        for dimension, value in self.dimensions.items():
            if dimension.sampler.__name__ == sp.flat_in_log.__name__:
                self.dimensions[dimension] = np.log10(value)
            elif dimension.sampler.__name__ == sp.uniform_in_sine.__name__:
                self.dimensions[dimension] = np.sin(value)
            elif dimension.sampler.__name__ == sp.uniform_in_cosine.__name__:
                self.dimensions[dimension] = np.cos(value + np.pi / 2)

    def calculate_prior_probability(self):
        p = 1
        for dimension, value in self.dimensions.items():
            p *= dimension.prior(dimension, value)
        return p



##############################################
#
# GridCallStroopwafel.py
#
# Python script meant to call STROOPWAFEL-interface.py for a variety of grid values. 
# This file calls interface.py, which calls COMPAS in a bunch of slurm scripts ##
# created 30-06-2020
# !! Many of these functions are adapted from compas_hpc_functions.py
#
##############################################
import numpy as np
import os
import time
from subprocess import Popen, PIPE
import sys
import pickle
import math
import shutil

SlurmJobStringTemplateWithEmail="""#!/bin/bash
#SBATCH --job-name=%s 			#job name
#SBATCH --nodes=%s 			#
#SBATCH --ntasks=%s 			# Number of cores
#SBATCH --partition=serial_requeue 	# Partition to submit to
#SBATCH --output=%s 			# output storage file
#SBATCH --error=%s 			# error storage file
#SBATCH --time=%s 			# Runtime in minutes
#SBATCH --mem=%s 			# Memory per cpu in MB (see also --mem-per-cpu)
#SBATCH --mail-user=%s 			# Send email to user
#SBATCH --mail-type=END			#
#
#Print some stuff on screen
echo $SLURM_JOB_ID
echo $SLURM_JOB_NAME
echo $SLURM_ARRAY_TASK_ID
#
#Set variables
export COMPAS_ROOT_DIR=%s
export SW_ROOT_DIR=%s
export BASE_DIR=%s
export OUT_DIR=%s
#
#copy relevant files
cp $SW_ROOT_DIR/GridCallStroopwafel_input.py $BASE_DIR/
cp $SW_ROOT_DIR/COMPAS_Output_Definitions.txt $BASE_DIR/
cp $SW_ROOT_DIR/randomSeed.txt $BASE_DIR/
#
#CD to folder
cd $SW_ROOT_DIR
#
# Run interface.py with command-line Flags
python $SW_ROOT_DIR/LiekeInterface.py --output_folder=$OUT_DIR --run_on_helios=True --num_systems=%s --num_per_core=%s --num_cores=%s --logfile-definition=$BASE_DIR/COMPAS_Output_Definitions.txt --compas_arg_flags="%s" --compas_arg_vals %s %s > $OUT_DIR/LiekeInterface.log 

cp $COMPAS_ROOT_DIR/CompasHPC/postProcessing.py  $OUT_DIR/
cp $COMPAS_ROOT_DIR/postProcessing/Folders/CosmicIntegration/PythonScripts_lieke/ComputeCIweights.py  $OUT_DIR/
"""
#--dependency=afterok:' + str(dependencyID)
# --compas_arg_flags="%s" --compas_arg_vals %s %s

PP_SlurmJobStringTemplate="""#!/bin/bash
#SBATCH --job-name=COMPAS_PP 		#job name
#SBATCH --nodes=1 			#
#SBATCH --ntasks=1 			# Number of cores
#SBATCH --partition=serial_requeue 	# Partition to submit to
#SBATCH --output=%s 			# output storage file
#SBATCH --error=%s 			# error storage file
#SBATCH --time=10:00:00 		# Runtime in minutes
#SBATCH --mem=0 			# Memory per cpu in MB (0 means unlimited)
#
echo $SLURM_JOB_ID
echo $SLURM_JOB_NAME
echo $SLURM_ARRAY_TASK_ID
#
#Set variables
export OUT_DIR=%s
cd $OUT_DIR
#Run postProcessing to combine output
python $OUT_DIR/postProcessing.py --masterFolderDir $OUT_DIR/masterFolder > $OUT_DIR/COMPAS_PP.log
"""
#

CI_SlurmJobStringTemplate="""#!/bin/bash
#SBATCH --job-name=COMPAS_CI 		#job name
#SBATCH --nodes=1 			#
#SBATCH --ntasks=1 			# Number of cores
#SBATCH --partition=serial_requeue 	# Partition to submit to
#SBATCH --output=%s 			# output storage file
#SBATCH --error=%s 			# error storage file
#SBATCH --time=10:00:00 		# Runtime in minutes
#SBATCH --mem=0 			# Memory per cpu in MB (0 means unlimited)
#
echo $SLURM_JOB_ID
echo $SLURM_JOB_NAME
echo $SLURM_ARRAY_TASK_ID
#
#Set variables
export OUT_DIR=%s
cd $OUT_DIR
#Run Coens cosmic integration to calculate CI weights
python $OUT_DIR/ComputeCIweights.py > $OUT_DIR/COMPAS_CI.log
"""
#

def runBashCommand(bashCommand, verbose=True):
	"""
	Run bash command
	Parameters
	-----------
	bashCommand : str
		Bash command to be run
	verbose : boolean
		Whether to echo command being run
	Returns
	--------
	"""
	if(verbose):
		print(bashCommand)
	os.system(bashCommand)
	return

###############################################
###		 MAIN      ###
###############################################
if __name__ == '__main__':
	# def main():
	###############################################
	# set variables
	###############################################
	job_name 		= "StroopGrid"
	number_of_nodes = 1
	number_of_cores = 50 
	walltime 		= "5:30:00"
	memory 			= 0#12000
	send_email		= True
	user_email 		= "aac.van.son@gmail.com"
	COMPAS_ROOT_DIR = "/n/home04/lvanson/Programs/COMPAS/" #"/Users/lieke/Documents/COMPAS/"
	SW_ROOT_DIR 	= "/n/home04/lvanson/Programs/stroopwafel/"#"/Users/lieke/surfdrive/Documents/CompareCOMPAS/Code/"

	#Parameters for interface.py
	I_num_systems	= int(1e5) # Number of systems per grid point
     # !! WARNING each batch has a max run time of 1 Hr and mem 1Gb
     # This corresponds to approx 3000 sys per core
	I_num_per_core	= 1000  # Number of systems run per core
	I_num_of_core	= 10	# number of cores/batches to run per grid point

	###############################################
	# Base outbut dir
	###############################################
	base_OUT_DIR			= "/n/de_mink_lab/Users/lvanson/CompasOutput/CItest_Default_N1e5"
	#"/Users/lieke/surfdrive/Documents/CompareCOMPAS/CompasOutput/SW/GridTest3x3/"#
	pklName='pickledGrid.pkl'
	verbose = True

	if os.path.exists(base_OUT_DIR):
		print('!!! REMOVING EXISTING DIR %s !!!' % base_OUT_DIR )
		shutil.rmtree(base_OUT_DIR)

	###############################################
	# Create Grid and Grid directories
	###############################################
	#  To see available options to change do 
	#  $COMPAS_ROOT_DIR/COMPAS/COMPAS --help
	gridDictionary = {}
	gridDictionary['--common-envelope-alpha'] = [1.0]#np.linspace(1.,10.,1) 
	gridDictionary['--wolf-rayet-multiplier'] = [1.0]#np.linspace(0.2,2,1) 
	dirNames = ['alpha', 'fWR']
	keys = list(gridDictionary.keys())
	I_flags			= "--common-envelope-alpha --wolf-rayet-multiplier"

	###############################################
	# !!! CURRENT STRUCTURE WORKS ONLY FOR 2D GRID !!!
	nGridPoints = 1
	for key in gridDictionary.keys():
		print(key, gridDictionary[key])
		nGridPoints *= len(gridDictionary[key])

	for j in range(nGridPoints):
		l = math.trunc(j/len(gridDictionary[keys[0]]) )
		k = j - (len(gridDictionary[keys[0]]) * l) 
		# print('k',k, 'l', l)
		alpha = gridDictionary[keys[0]][k] 
		fWR = gridDictionary[keys[1]][l]
		gridPointDir = dirNames[0]+str(alpha)+'_'+dirNames[1]+str(fWR)
		print('Making ', gridPointDir)

		OUT_DIR			= base_OUT_DIR + gridPointDir +'/' #"/n/de_mink_lab/Users/lvanson/CompasOutput/StroopwafelTest/GridTest/"
		outfile 		= OUT_DIR + "StroopGrid.out"
		errfile 		= OUT_DIR + "StroopGrid.err"


		###############################################
		# Make the output directory
		###############################################
		if not os.path.exists(OUT_DIR):
			os.makedirs(OUT_DIR)

		###############################################
		# Generate Slurm Job String for interface.py
		###############################################
		interface_job_string = SlurmJobStringTemplateWithEmail % (job_name, number_of_nodes, number_of_cores, outfile, errfile, walltime, memory, user_email,COMPAS_ROOT_DIR, SW_ROOT_DIR, base_OUT_DIR, OUT_DIR, I_num_systems, I_num_per_core, I_num_of_core, I_flags, alpha, fWR)
		#
		# print(interface_job_string)
		# Save to a file
		sbatchFile = open(OUT_DIR+'/SW_Slurm.sbatch','w')
		sbatchFile.write(interface_job_string)
		sbatchFile.close()

		###############################################
		# Send SW command to bash
		###############################################	
		sbatchArrayCommand = 'sbatch ' + os.path.join(OUT_DIR+'/SW_Slurm.sbatch') 
		# Open a pipe to the sbatch command.
		proc = Popen(sbatchArrayCommand, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE, close_fds=True)
		## Send job_string to sbatch
		if (sys.version_info > (3, 0)):
			proc.stdin.write(sbatchArrayCommand.encode('utf-8'))
		else:
			proc.stdin.write(sbatchArrayCommand)

		print('sbatchArrayCommand', sbatchArrayCommand)
		out, err = proc.communicate()
		print("out = ", out)
		main_job_id = out.split()[-1]
		print("main_job_id", main_job_id)

		###############################################
		# Generate Slurm Job String for postprocessing
		###############################################
		PPoutfile 		= OUT_DIR + "COMPAS_PP.out"
		PPerrfile 		= OUT_DIR + "COMPAS_PP.err"
		PP_job_string = PP_SlurmJobStringTemplate % (PPoutfile, PPerrfile, OUT_DIR)
		# print(PP_job_string)
		# Save to a file
		PPsbatchFile = open(OUT_DIR+'/postProcessing.sbatch','w')
		PPsbatchFile.write(PP_job_string)
		PPsbatchFile.close()

		###############################################
		# Send PP command to bash with afterok main COMPAS
		###############################################	        
		sbatchPPCommand = 'sbatch --dependency=afterok:' + str(int(main_job_id)) + ' ' + os.path.join(OUT_DIR+'/postProcessing.sbatch') 
		# Open a pipe to the sbatch command.
		proc = Popen(sbatchPPCommand, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE, close_fds=True)
		# Send job_string to sbatch
		if (sys.version_info > (3, 0)):
		        proc.stdin.write(sbatchPPCommand.encode('utf-8'))
		else:
		        proc.stdin.write(sbatchPPCommand)

		print('sbatchPPCommand', sbatchPPCommand)
		ppOut, pperr = proc.communicate()
		print("ppOut = ", ppOut)
		print("err = ", pperr)
		ppJobID = ppOut.split()[-1]
		print("Post-processing job ID = ", ppJobID)


		###################################################
		# Generate Slurm Job String for Cosmic integration
		###################################################
		CIoutfile 		= OUT_DIR + "COMPAS_CI.out"
		CIerrfile 		= OUT_DIR + "COMPAS_CI.err"
		CI_job_string   = CI_SlurmJobStringTemplate % (CIoutfile, CIerrfile, OUT_DIR)
		# print(PP_job_string)
		# Save to a file
		CIsbatchFile = open(OUT_DIR+'/CosmicIntegration.sbatch','w')
		CIsbatchFile.write(CI_job_string)
		CIsbatchFile.close()

		##########################################################
		# Send Cosmic integration command to bash with afterok PP
		##########################################################        
		sbatchCICommand = 'sbatch --dependency=afterok:' + str(int(ppJobID)) + ' ' + os.path.join(OUT_DIR+'/CosmicIntegration.sbatch') 
		# Open a pipe to the sbatch command.
		proc = Popen(sbatchCICommand, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE, close_fds=True)
		# Send job_string to sbatch
		if (sys.version_info > (3, 0)):
		        proc.stdin.write(sbatchPPCommand.encode('utf-8'))
		else:
		        proc.stdin.write(sbatchPPCommand)

		print('sbatchPPCommand', sbatchPPCommand)
		CIOut, CIerr = proc.communicate()
		print("ppOut = ", CIOut)
		print("err = ", CIerr)
		CIJobID = CIOut.split()[-1]
		print("Cosmic integration job ID = ", CIJobID)


		###############################################
		# Run bash command (OLD)
		###############################################
		# runBashCommand(interface_job_string, verbose=False)





#!/usr/bin/env python

import os
import pandas as pd
import shutil
import time
import numpy as np
from stroopwafel import *
import argparse

parser=argparse.ArgumentParser()
parser.add_argument('--num_systems', help = 'Total number of systems', type = int, default = 100)#100000)
parser.add_argument('--num_cores', help = 'Number of cores to run in parallel', type = int, default = 10)#50)
parser.add_argument('--num_per_core', help = 'Number of systems to generate in one core', type = int, default = 10)#1000)
parser.add_argument('--debug', help = 'If debug of COMPAS is to be printed', type = bool, default = True)
parser.add_argument('--mc_only', help = 'If run in MC simulation mode only', type = bool, default = False)
parser.add_argument('--run_on_helios', help = 'If we are running on helios (or other slurm) nodes', type = bool, default = False)
parser.add_argument('--output_filename', help = 'Output filename', default = 'samples.csv')
parser.add_argument('--output_folder', help = 'output folder', default = './output')
parser.add_argument('--compas_arg_flags', '--cf', nargs='*', help='flag names of aditional compas arguments', required=False, default = None)
parser.add_argument('--compas_arg_vals', '--cv', nargs='*', help='aditional compas arguments', required=False, type=float, default = None)
namespace, extra_params = parser.parse_known_args()
if not np.logical_or(np.logical_and(not namespace.compas_arg_flags, not namespace.compas_arg_vals), np.logical_and(namespace.compas_arg_flags != None, namespace.compas_arg_vals != None) ):
    parser.error("Both --compas_arg_flags and --compas_arg_vals have to be defined simultaniously!")

# STEP 2 : Define the functions
def create_dimensions():
    """
    This Function that will create all the dimensions for stroopwafel, a dimension is basically one of the variables you want to sample
    Invoke the Dimension class to create objects for each variable. Look at the Dimension class definition in classes.py for more.
    It takes the name of the dimension, its max and min value. 
    The Sampler class will tell how to sample this dimension. Similarly, prior tells it how it calculates the prior. You can find more of these in their respective modules
    OUT:
        As Output, this should return a list containing all the instances of Dimension class.
    """
    m1 = classes.Dimension('Mass_1', 8, 150, sampler.kroupa, prior.kroupa)
    q = classes.Dimension('q', 0.01, 1, sampler.uniform, prior.uniform, should_print = False)
    metallicity_1 = classes.Dimension('Metallicity_1', 1e-4, 0.023, sampler.flat_in_log, prior.flat_in_log)
    a = classes.Dimension('Separation', 0.01, 1000, sampler.flat_in_log, prior.flat_in_log)
    #kick_velocity_random_1 = classes.Dimension('Kick_Velocity_Random_1', 0, 1, sampler.uniform, prior.uniform)
    #kick_theta_1 = classes.Dimension('Kick_Theta_1', -np.pi / 2, np.pi / 2, sampler.uniform_in_sine, prior.uniform_in_sine)
    #kick_phi_1 = classes.Dimension('Kick_Phi_1', 0, 2 * np.pi, sampler.uniform, prior.uniform)
    #kick_velocity_random_2 = classes.Dimension('Kick_Velocity_Random_2', 0, 1, sampler.uniform, prior.uniform)
    #kick_theta_2 = classes.Dimension('Kick_Theta_2', -np.pi / 2, np.pi / 2, sampler.uniform_in_sine, prior.uniform_in_sine)
    #kick_phi_2 = classes.Dimension('Kick_Phi_2', 0, 2 * np.pi, sampler.uniform, prior.uniform)
    #return [m1, q, a, kick_velocity_random_1, kick_theta_1, kick_phi_1, kick_velocity_random_2, kick_theta_2, kick_phi_2]
    return [m1, q, metallicity_1, a]


def update_properties(locations, dimensions):
    """
    This function is not mandatory, it is required only if you have some dependent variable. 
    For example, if you want to sample Mass_1 and q, then Mass_2 is a dependent variable which is product of the two.
    Similarly, you can assume that Metallicity_2 will always be equal to Metallicity_1
    IN:
        locations (list(Location)) : A list containing objects of Location class in classes.py. 
        You can play with them and update whatever fields you like or add more in the property (which is a dictionary)
    OUT: Not Required
    """
    m1 = dimensions[0]
    q = dimensions[1]
    metallicity_1 = dimensions[2]
    for location in locations:
        location.properties['Mass_2'] = location.dimensions[m1] * location.dimensions[q]
        location.properties['Metallicity_2'] = location.dimensions[metallicity_1]
        # location.properties['Metallicity_2'] = location.properties['Metallicity_1'] = constants.METALLICITY_SOL
        location.properties['Eccentricity'] = 0
        #location.properties['Kick_Mean_Anomaly_1'] = np.random.uniform(0, 2 * np.pi, 1)[0]
        #location.properties['Kick_Mean_Anomaly_2'] = np.random.uniform(0, 2 * np.pi, 1)[0]

def get_default_COMPAS_commands():
    """
    This function loads the default values from $COMPAS_ROOT_DIR/defaults/pythonSubmitDefault.py
    IN:
        
    OUT: default_commandlineOptions (list(String)) : Will be added to compas_args. 
    """
    #Add the COMPAS root dir to the import envir
    import sys
    compas_root = os.environ["COMPAS_ROOT_DIR"]
    sys.path.append(compas_root+'/defaults/')
    #Import the pythonSubmitDefault file (can be adjusted to any pythonSubmit.py file)
    import pythonSubmitDefault as psd

    #-- Get the program options
    programOptions =psd.pythonProgramOptions()
    booleanChoices = programOptions.booleanChoices()
    booleanCommands = programOptions.booleanCommands()

    numericalChoices = programOptions.numericalChoices()
    numericalCommands = programOptions.numericalCommands()

    stringChoices = programOptions.stringChoices()
    stringCommands = programOptions.stringCommands()

    listChoices = programOptions.listChoices()
    listCommands = programOptions.listCommands()

    execute = ''

    # create a list of default command line options
    default_commandlineOptions = psd.generateCommandLineOptions(execute,booleanChoices,booleanCommands,numericalChoices,numericalCommands,stringChoices,stringCommands,listChoices,listCommands)
    default_commandlineOptions = default_commandlineOptions.split()

    return default_commandlineOptions

def get_extra_arg():
    """
    Reshape the compas_arg_flags and compas_arg_vals from the ArgumentParser to a list that can be added to the final compas_args
    IN: 
    OUT: extra_arg (list(String)) : Will be added to compas_args.  
    """
    extra_arg = []
    if np.logical_and(namespace.compas_arg_flags != None, namespace.compas_arg_vals != None):
        try: 
            additional_args = namespace.compas_arg_flags[0].split()
        except:
            additional_args = namespace.compas_arg_flags.split()        
        additional_values = namespace.compas_arg_vals
        if len(additional_args) != len(additional_values):
            print("currently len(compas_arg_flags)=%s and len(compas_arg_vals)=%s" % (len(additional_args), len(additional_values)))
            raise ValueError("you have to provide an equal number of compas_arg_flags and compas_arg_vals!")
        extra_arg = list(np.array([(additional_args[i], additional_values[i]) for i in range(len(additional_args)) ]).flat)
    return extra_arg

def check_duplicates(default_args, input_arg):
    """
    Create an adjusted default list that does not contain the flags in compas_arg_flags (to avoid duplicity)
    IN:  default_args = default_commandlineOptions from COMPAS, input_arg  = sw + ArgumentParser input flags
    OUT:  adjusted_default (list(String))
    """
    adjusted_default = default_args.copy()
    remove_from_default = ["--number-of-binaries"] 
    for arg in input_arg + remove_from_default:
        #print('arg', arg)
        try:
            if np.logical_and(arg.startswith('--'), arg in adjusted_default):
                problem_i = np.where(np.array(adjusted_default) == arg)[0][0]
                if not adjusted_default[problem_i+1].startswith('--'):
                    #print("Remove both the flag and the option", "[problem_i]", [problem_i], "[[problem_i+1]", [problem_i+1], adjusted_default[problem_i], adjusted_default[problem_i+1])
                    #Remove both the flag and the option
                    del adjusted_default[problem_i+1]#Always delete last one first!
                    del adjusted_default[problem_i]
                else:
                    #only remove the flag (e.g. for boolean)
                    #print("only remove the flag (e.g. for boolean)", "[problem_i]", [problem_i], "[[problem_i+1]", [problem_i+1], adjusted_default[problem_i])
                    del adjusted_default[problem_i]
        except:
            #print('this is not a flag: %s, continue' % arg)
            continue
    return adjusted_default




def configure_code_run(batch):
    """
    This function tells stroopwafel what program to run, along with its arguments.
    IN:
        batch(dict): This is a dictionary which stores some information about one of the runs. It has an number key which stores the unique id of the run
            It also has a subprocess which will run under the key process. Rest, it depends on the user. User is free to store any information they might need later 
            for each batch run in this dictionary. For example, here I have stored the 'output_container' and 'grid_filename' so that I can read them during discovery of interesting systems below
    OUT:
        compas_args (list(String)) : This defines what will run. It should point to the executable file along with the arguments.
        Additionally one must also store the grid_filename in the batch so that the grid file is created
    """
    batch_num = batch['number']
    randomSeedFileName = 'RandomSeed.txt'
    if os.path.isfile(output_folder + randomSeedFileName):
        #print('found randomseed file', output_folder + randomSeedFileName )
        seeds = np.loadtxt(output_folder + randomSeedFileName)
        current_seed = int(seeds[batch_num])
    else:
        current_seed = np.random.randint(2, 2**63 - 1)
    #print('current_seed', current_seed)

    grid_filename = output_folder + '/grid_' + str(batch_num) + '.csv'
    output_container = 'batch_' + str(batch_num)
    #get and combine commandline arguments
    extra_arg           = get_extra_arg()
    sw_args             = [compas_executable, "--grid", grid_filename, '--outputPath', output_folder, '--output-container', output_container, '--random-seed', current_seed]
    input_arg           = sw_args + extra_arg

    default_args        = get_default_COMPAS_commands()
    #Check for duplicate arguments between default and user input
    adjusted_default    = check_duplicates(default_args, input_arg)

    compas_args         = input_arg + adjusted_default
    for params in extra_params:
        compas_args.extend(params.split("="))
    #print('Run_details', compas_args)    
    batch['grid_filename'] = grid_filename
    batch['output_container'] = output_container
    return compas_args


def interesting_systems(batch):
    """
    This is a mandatory function, it tells stroopwafel what an interesting system is. User is free to define whatever looks interesting to them.
    IN:
        batch (dict): As input you will be given the current batch which just finished its execution. You can take in all the keys you defined in the configure_code_run method above
    OUT:
        Number of interesting systems
        In the below example, I define all the NSs as interesting, so I read the files, get the SEED from the system_params file and define the key is_hit in the end for all interesting systems 
    """
    try:
        #What will be written as output to the .csv file wiill be determined here
        folder = os.path.join(output_folder, batch['output_container'])
        shutil.move(batch['grid_filename'], folder + '/grid_' + str(batch['number']) + '.csv')
        system_parameters = pd.read_csv(folder + '/BSE_System_Parameters.csv', skiprows = 2)
        system_parameters.rename(columns = lambda x: x.strip(), inplace = True)
        seeds = system_parameters['SEED']
        for index, sample in enumerate(batch['samples']):
            seed = seeds[index]
            sample.properties['SEED'] = seed
            sample.properties['is_hit'] = 0
            sample.properties['batch'] = batch['number']
        double_compact_objects = pd.read_csv(folder + '/BSE_Double_Compact_Objects.csv', skiprows = 2)
        double_compact_objects.rename(columns = lambda x: x.strip(), inplace = True)
        #Generally, this is the line you would want to change.
        dbh = double_compact_objects[np.logical_and(double_compact_objects['Merges_Hubble_Time'] == 1, \
            np.logical_and(double_compact_objects['Stellar_Type_1'] == 14, double_compact_objects['Stellar_Type_2'] == 14))]
        interesting_systems_seeds = set(dbh['SEED'])
        for sample in batch['samples']:
            if sample.properties['SEED'] in interesting_systems_seeds:
                sample.properties['is_hit'] = 1
        return len(dbh)
    except IOError as error:
        return 0

def selection_effects(sw):
    """
    This is not a mandatory function, it was written to support selection effects
    Fills in selection effects for each of the distributions
    IN:
        sw (Stroopwafel) : Stroopwafel object
    """
    if hasattr(sw, 'adapted_distributions'):
        biased_masses = []
        rows = []
        for distribution in sw.adapted_distributions:
            folder = os.path.join(output_folder, 'batch_' + str(int(distribution.mean.properties['batch'])))
            dco_file = pd.read_csv(folder + '/BSE_Double_Compact_Objects.csv', skiprows = 2)
            dco_file.rename(columns = lambda x: x.strip(), inplace = True)
            row = dco_file.loc[dco_file['SEED'] == distribution.mean.properties['SEED']]
            rows.append([row.iloc[0]['Mass_1'], row.iloc[0]['Mass_2']])
            biased_masses.append(np.power(max(rows[-1]), 2.2))
        # update the weights
        mean = np.mean(biased_masses)
        for index, distribution in enumerate(sw.adapted_distributions):
            distribution.biased_weight = np.power(max(rows[index]), 2.2) / mean

def rejected_systems(locations, dimensions):
    """
    This method takes a list of locations and marks the systems which can be
    rejected by the prior distribution
    IN:
        locations (List(Location)): list of location to inspect and mark rejected
    OUT:
        num_rejected (int): number of systems which can be rejected
    """
    m1 = dimensions[0]
    q = dimensions[1]
    metallicity_1 = dimensions[2]
    a = dimensions[3]
    [location.properties.update({'Mass_2': location.dimensions[m1] * location.dimensions[q]}) for location in locations]
    mass_1 = [location.dimensions[m1] for location in locations]
    mass_2 = [location.properties['Mass_2'] for location in locations]
    Z_1 = [location.dimensions[metallicity_1] for location in locations]
    Z_2 = Z_1
    # metallicity_1 = [location.properties['Metallicity_1'] for location in locations]
    # metallicity_2 = [location.properties['Metallicity_2'] for location in locations]
    eccentricity = [location.properties['Eccentricity'] for location in locations]
    num_rejected = 0
    for index, location in enumerate(locations):
        radius_1 = utils.get_zams_radius(mass_1[index], Z_1[index])
        radius_2 = utils.get_zams_radius(mass_2[index], Z_2[index])
        roche_lobe_tracker_1 = radius_1 / (location.dimensions[a] * (1 - eccentricity[index]) * utils.calculate_roche_lobe_radius(mass_1[index], mass_2[index]))
        roche_lobe_tracker_2 = radius_2 / (location.dimensions[a] * (1 - eccentricity[index]) * utils.calculate_roche_lobe_radius(mass_2[index], mass_1[index]))
        location.properties['is_rejected'] = 0
        if (mass_2[index] < constants.MINIMUM_SECONDARY_MASS) or (location.dimensions[a] <= (radius_1 + radius_2)) \
        or roche_lobe_tracker_1 > 1 or roche_lobe_tracker_2 > 1:
            location.properties['is_rejected'] = 1
            num_rejected += 1
    return num_rejected

if __name__ == '__main__':
    start_time = time.time()
    #Define the parameters to the constructor of stroopwafel
    TOTAL_NUM_SYSTEMS = namespace.num_systems #total number of systems you want in the end
    NUM_CPU_CORES = namespace.num_cores #Number of cpu cores you want to run in parellel
    NUM_SYSTEMS_PER_RUN = namespace.num_per_core #Number of systems generated by each of run on each cpu core
    debug = namespace.debug #If True, will print the logs given by the external program (like COMPAS)
    run_on_helios = namespace.run_on_helios #If True, it will run on a clustered system helios, rather than your pc
    mc_only = namespace.mc_only # If you dont want to do the refinement phase and just do random mc exploration
    output_filename = namespace.output_filename #The name of the output file
    compas_executable = os.path.join(os.environ.get('COMPAS_ROOT_DIR'), 'src/COMPAS') # Location of the executable
    output_folder =  namespace.output_folder  #os.path.join(os.getcwd(), 'output') # Folder you want to receieve outputs, here the current working directory, but you can specify anywhere

    # Make the output directory
    # You already made the folder if you are running a grid of values
    Gridcall = True 
    if not os.path.exists(output_folder):
    	os.makedirs(output_folder)
    else:
        if not Gridcall:
            shutil.rmtree(output_folder)


    print('TOTAL_NUM_SYSTEMS', TOTAL_NUM_SYSTEMS, 'NUM_CPU_CORES', NUM_CPU_CORES, 'NUM_SYSTEMS_PER_RUN', NUM_SYSTEMS_PER_RUN, 'run_on_helios', run_on_helios)
    # STEP 1 : Create an instance of the Stroopwafel class
    sw = stroopwafel.Stroopwafel(TOTAL_NUM_SYSTEMS, NUM_CPU_CORES, NUM_SYSTEMS_PER_RUN, output_folder, output_filename, debug = debug, run_on_helios = run_on_helios, mc_only = mc_only)

    #STEP 3: Initialize the stroopwafel object with the user defined functions and create dimensions and initial distribution
    dimensions = create_dimensions()
    sw.initialize(dimensions, interesting_systems, configure_code_run, rejected_systems, update_properties_method = update_properties)

    intial_pdf = distributions.InitialDistribution(dimensions)
    #STEP 4: Run the 4 phases of stroopwafel
    sw.explore(intial_pdf) #Pass in the initial distribution for exploration phase
    now = time.time()

    sw.adapt(n_dimensional_distribution_type = distributions.Gaussian) #Adaptaion phase, tell stroopwafel what kind of distribution you would like to create instrumental distributions
    ## Do selection effects
    #selection_effects(sw)
    sw.refine() #Stroopwafel will draw samples from the adapted distributions
    sw.postprocess(distributions.Gaussian, only_hits = False) #Run it to create weights, if you want only hits in the output, then make only_hits = True

    end_time = time.time()
    print ("Total running time = %d seconds" %(end_time - start_time))
